@@ -16,19 +16,22 @@
 * limitations under the License.
 */
 
-//#ifndef NDEBUG
+#ifndef NDEBUG
 //#define PRINT_DEBUG
-//#endif
+#endif
 
-#include "datamatrix/DMDetector.h"
+#include "DMDetector.h"
 #include "BitMatrix.h"
-#include "DecodeStatus.h"
+#include "BitMatrixCursor.h"
 #include "DetectorResult.h"
+#include "ResultPoint.h"
 #include "GridSampler.h"
+#include "Point.h"
 #include "WhiteRectDetector.h"
 
 #ifdef PRINT_DEBUG
-#include "ByteMatrix.h"
+#include "BitMatrixIO.h"
+#include "LogMatrix.h"
 #endif
 
 #include <algorithm>
@@ -38,8 +41,17 @@
 #include <cstdlib>
 #include <map>
 #include <numeric>
+#include <utility>
+#include <vector>
 
 namespace ZXing {
+
+#ifdef PRINT_DEBUG
+static LogMatrix log;
+#else
+template<typename T> void log(PointT<T>, int = 0) {}
+#endif
+
 namespace DataMatrix {
 
 /**
@@ -112,11 +124,7 @@ inline static bool IsValidPoint(const ResultPoint& p, int imgWidth, int imgHeigh
 
 inline static float RoundToNearest(float x)
 {
-#if defined(__ANDROID__) && defined(__GNUC__)
-	return static_cast<float>(round(x));
-#else
 	return std::round(x);
-#endif
 }
 
 /**
@@ -128,15 +136,15 @@ static bool CorrectTopRightRectangular(const BitMatrix& image, const ResultPoint
 									   const ResultPoint& topRight, int dimensionTop, int dimensionRight,
 									   ResultPoint& result)
 {
-	float corr = RoundToNearest(ResultPoint::Distance(bottomLeft, bottomRight)) / static_cast<float>(dimensionTop);
-	float norm = RoundToNearest(ResultPoint::Distance(topLeft, topRight));
+	float corr = RoundToNearest(distance(bottomLeft, bottomRight)) / static_cast<float>(dimensionTop);
+	float norm = RoundToNearest(distance(topLeft, topRight));
 	float cos = (topRight.x() - topLeft.x()) / norm;
 	float sin = (topRight.y() - topLeft.y()) / norm;
 
 	ResultPoint c1(topRight.x() + corr*cos, topRight.y() + corr*sin);
 
-	corr = RoundToNearest(ResultPoint::Distance(bottomLeft, topLeft)) / (float)dimensionRight;
-	norm = RoundToNearest(ResultPoint::Distance(bottomRight, topRight));
+	corr = RoundToNearest(distance(bottomLeft, topLeft)) / (float)dimensionRight;
+	norm = RoundToNearest(distance(bottomRight, topRight));
 	cos = (topRight.x() - bottomRight.x()) / norm;
 	sin = (topRight.y() - bottomRight.y()) / norm;
 
@@ -170,15 +178,15 @@ static bool CorrectTopRightRectangular(const BitMatrix& image, const ResultPoint
 static ResultPoint CorrectTopRight(const BitMatrix& image, const ResultPoint& bottomLeft, const ResultPoint& bottomRight,
                                    const ResultPoint& topLeft, const ResultPoint& topRight, int dimension)
 {
-	float corr = RoundToNearest(ResultPoint::Distance(bottomLeft, bottomRight)) / (float)dimension;
-	float norm = RoundToNearest(ResultPoint::Distance(topLeft, topRight));
+	float corr = RoundToNearest(distance(bottomLeft, bottomRight)) / (float)dimension;
+	float norm = RoundToNearest(distance(topLeft, topRight));
 	float cos = (topRight.x() - topLeft.x()) / norm;
 	float sin = (topRight.y() - topLeft.y()) / norm;
 
 	ResultPoint c1(topRight.x() + corr * cos, topRight.y() + corr * sin);
 
-	corr = RoundToNearest(ResultPoint::Distance(bottomLeft, topLeft)) / (float)dimension;
-	norm = RoundToNearest(ResultPoint::Distance(bottomRight, topRight));
+	corr = RoundToNearest(distance(bottomLeft, topLeft)) / (float)dimension;
+	norm = RoundToNearest(distance(bottomRight, topRight));
 	cos = (topRight.x() - bottomRight.x()) / norm;
 	sin = (topRight.y() - bottomRight.y()) / norm;
 
@@ -199,29 +207,11 @@ static ResultPoint CorrectTopRight(const BitMatrix& image, const ResultPoint& bo
 	return l1 <= l2 ? c1 : c2;
 }
 
-static BitMatrix SampleGrid(const BitMatrix& image, const ResultPoint& topLeft, const ResultPoint& bottomLeft,
-							const ResultPoint& bottomRight, const ResultPoint& topRight, int dimensionX, int dimensionY)
+static DetectorResult SampleGrid(const BitMatrix& image, const ResultPoint& topLeft, const ResultPoint& bottomLeft,
+								 const ResultPoint& bottomRight, const ResultPoint& topRight, int width, int height)
 {
-	return GridSampler::Instance()->sampleGrid(
-		image,
-		dimensionX,
-		dimensionY,
-		0.5f,
-		0.5f,
-		dimensionX - 0.5f,
-		0.5f,
-		dimensionX - 0.5f,
-		dimensionY - 0.5f,
-		0.5f,
-		dimensionY - 0.5f,
-		topLeft.x(),
-		topLeft.y(),
-		topRight.x(),
-		topRight.y(),
-		bottomRight.x(),
-		bottomRight.y(),
-		bottomLeft.x(),
-		bottomLeft.y());
+	return SampleGrid(image, width, height,
+					  {Rectangle(width, height, 0.5), {topLeft, topRight, bottomRight, bottomLeft}});
 }
 
 /**
@@ -241,9 +231,9 @@ static float CrossProductZ(const ResultPoint& a, const ResultPoint& b, const Res
 static void OrderByBestPatterns(const ResultPoint*& p0, const ResultPoint*& p1, const ResultPoint*& p2)
 {
 	// Find distances between pattern centers
-	float zeroOneDistance = ResultPoint::Distance(*p0, *p1);
-	float oneTwoDistance = ResultPoint::Distance(*p1, *p2);
-	float zeroTwoDistance = ResultPoint::Distance(*p0, *p2);
+	float zeroOneDistance = distance(*p0, *p1);
+	float oneTwoDistance = distance(*p1, *p2);
+	float zeroTwoDistance = distance(*p0, *p2);
 
 	const ResultPoint* pointA;
 	const ResultPoint* pointB;
@@ -424,11 +414,7 @@ static DetectorResult DetectOld(const BitMatrix& image)
 		dimensionTop = dimensionRight = dimension;
 	}
 
-	auto bits = SampleGrid(image, *topLeft, *bottomLeft, *bottomRight, correctedTopRight, dimensionTop, dimensionRight);
-	if (bits.empty())
-		return {};
-
-	return {std::move(bits), {*topLeft, *bottomLeft, *bottomRight, correctedTopRight}};
+	return SampleGrid(image, *topLeft, *bottomLeft, *bottomRight, correctedTopRight, dimensionTop, dimensionRight);
 }
 
 /**
@@ -439,80 +425,15 @@ static DetectorResult DetectOld(const BitMatrix& image)
 *  * works with real-world codes that have just one module wide quite-zone (which is perfectly in spec)
 */
 
-template <typename T> struct PointT
-{
-	T x = 0, y = 0;
-	PointT() = default;
-	PointT(T x, T y) : x(x), y(y) {}
-	template <typename U>
-	explicit PointT(const PointT<U>& p) : x((T)p.x), y((T)p.y) {}
-	explicit PointT(const ResultPoint& p) : x(p.x()), y(p.y()) {}
-	operator ResultPoint() const { return {static_cast<float>(x), static_cast<float>(y)}; }
-};
-
-template <typename T> bool operator == (const PointT<T>& a, const PointT<T>& b)
-{
-	return a.x == b.x && a.y == b.y;
-}
-
-template <typename T> bool operator != (const PointT<T>& a, const PointT<T>& b)
-{
-	return !(a == b);
-}
-
-template <typename T, typename U> auto operator + (const PointT<T>& a, const PointT<U>& b) -> PointT<decltype(a.x + b.x)>
-{
-	return {a.x + b.x, a.y + b.y};
-}
-
-template <typename T, typename U> auto operator - (const PointT<T>& a, const PointT<U>& b) -> PointT<decltype(a.x - b.x)>
-{
-	return {a.x - b.x, a.y - b.y};
-}
-
-template <typename T, typename U> PointT<T> operator * (U s, const PointT<T>& a)
-{
-	return {s * a.x, s * a.y};
-}
-
-template <typename T, typename U> PointT<T> operator / (const PointT<T>& a, U d)
-{
-	return {a.x / d, a.y / d};
-}
-
-template <typename T, typename U> double operator * (const PointT<T>& a, const PointT<U>& b)
-{
-	return double(a.x) * b.x + a.y * b.y;
-}
-
-template <typename T> double distance(PointT<T> a, PointT<T> b)
-{
-	auto d = a - b;
-	return std::sqrt(d * d);
-}
-
-using PointI = PointT<int>;
-using PointF = PointT<double>;
-
-template <typename T> PointF normalized(PointT<T> a)
-{
-	return PointF(a) / distance(a, {});
-}
-
-PointI round(PointF p)
-{
-    return PointI(::lround(p.x), ::lround(p.y));
-}
-
 class RegressionLine
 {
-	std::vector<PointI> _points;
+	std::vector<PointF> _points;
 	PointF _directionInward;
 	double a = NAN, b = NAN, c = NAN;
 
 	friend PointF intersect(const RegressionLine& l1, const RegressionLine& l2);
 
-	bool evaluate(const std::vector<PointI> ps)
+	bool evaluate(const std::vector<PointF>& ps)
 	{
 		auto mean = std::accumulate(ps.begin(), ps.end(), PointF()) / ps.size();
 		double sumXX = 0, sumYY = 0, sumXY = 0;
@@ -528,12 +449,12 @@ class RegressionLine
 			a = +sumXY / std::sqrt(sumXX * sumXX + sumXY * sumXY);
 			b = -sumXX / std::sqrt(sumXX * sumXX + sumXY * sumXY);
 		}
-		if (_directionInward * normal() < 0) {
+		if (dot(_directionInward, normal()) < 0) {
 			a = -a;
 			b = -b;
 		}
-		c = normal() * mean; // (a*mean.x + b*mean.y);
-		return _directionInward * normal() > 0.5; // angle between original and new direction is at most 60 degree
+		c = dot(normal(), mean); // (a*mean.x + b*mean.y);
+		return dot(_directionInward, normal()) > 0.5; // angle between original and new direction is at most 60 degree
 	}
 
 	template <typename Container, typename Filter>
@@ -550,20 +471,20 @@ class RegressionLine
 	}
 
 public:
-	const std::vector<PointI>& points() const { return _points; }
+	const std::vector<PointF>& points() const { return _points; }
 	int length() const { return _points.size() >= 2 ? int(distance(_points.front(), _points.back())) : 0; }
 	bool isValid() const { return !std::isnan(a); }
 	PointF normal() const { return isValid() ? PointF(a, b) : _directionInward; }
-	double signedDistance(PointI p) const { return normal() * p - c; }
-	PointF project(PointI p) const { return p - signedDistance(p) * normal(); }
+	double signedDistance(PointF p) const { return dot(normal(), p) - c; }
+	PointF project(PointF p) const { return p - signedDistance(p) * normal(); }
 
 	void reverse() { std::reverse(_points.begin(), _points.end()); }
 
-	void add(PointI p) {
+	void add(PointF p) {
 		assert(_directionInward != PointF());
 		_points.push_back(p);
 		if (_points.size() == 1)
-			c = normal() * p;
+			c = dot(normal(), p);
 	}
 
 	void pop_back() { _points.pop_back(); }
@@ -579,7 +500,7 @@ public:
 			while (true) {
 				old_points_size = _points.size();
 				_points.erase(std::remove_if(_points.begin(), _points.end(),
-											 [this, maxDist](PointI p) { return this->signedDistance(p) > maxDist; }),
+											 [this, maxDist](auto p) { return this->signedDistance(p) > maxDist; }),
 							  _points.end());
 				if (old_points_size == _points.size())
 					break;
@@ -621,12 +542,12 @@ public:
 		auto lineLength = distance(beg, end) - unitPixelDist;
 		auto meanGapSize = lineLength / gapSizes.size();
 #ifdef PRINT_DEBUG
-		printf("unit pixel dist: %f\n", unitPixelDist);
-		printf("lineLength: %f, meanGapSize: %f, gaps: %lu\n", lineLength, meanGapSize, gapSizes.size());
+		printf("unit pixel dist: %.1f\n", unitPixelDist);
+		printf("lineLength: %.1f, meanGapSize: %.1f, gaps: %lu\n", lineLength, meanGapSize, gapSizes.size());
 #endif
 		meanGapSize = average(gapSizes, [&](double dist){ return std::abs(dist - meanGapSize) < meanGapSize/2; });
 #ifdef PRINT_DEBUG
-		printf("lineLength: %f, meanGapSize: %f, gaps: %lu\n", lineLength, meanGapSize, gapSizes.size());
+		printf("lineLength: %.1f, meanGapSize: %.1f, gaps: %lu\n", lineLength, meanGapSize, gapSizes.size());
 #endif
 		return lineLength / meanGapSize;
 	}
@@ -642,53 +563,10 @@ PointF intersect(const RegressionLine& l1, const RegressionLine& l2)
 	return {x, y};
 }
 
-class EdgeTracer
+class EdgeTracer : public BitMatrixCursorF
 {
-	const BitMatrix& image;
-	PointI p; // current position
-	PointF d; // current direction
-
-	static PointF mainDirection(PointF d)
-	{
-		assert(std::abs(d.x) != std::abs(d.y));
-		return std::abs(d.x) > std::abs(d.y) ? PointF(d.x, 0) : PointF(0, d.y);
-	}
-
 	enum class StepResult { FOUND, OPEN_END, CLOSED_END };
 
-	bool isIn(PointI p) const
-	{
-		const int b = 0;
-		return  b <= p.x && p.x < image.width()-b &&
-		        b <= p.y && p.y < image.height()-b;
-	}
-	bool isIn(PointF p) const { return isIn(round(p)); }
-	bool isIn() const { return isIn(p); }
-
-	class Value
-	{
-		enum { INVALID, WHITE, BLACK };
-		int v = INVALID;
-	public:
-		Value() = default;
-		Value(bool isBlack) : v(isBlack ? BLACK : WHITE) {}
-		bool isValid() const { return v != INVALID; }
-		bool isWhite() const { return v == WHITE; }
-		bool isBlack() const { return v == BLACK; }
-	};
-
-	Value getAt(PointF p) const
-	{
-		auto q = round(p);
-		if (!isIn(q))
-			return {};
-		return {image.get(q.x, q.y)};
-	}
-
-	bool blackAt(PointF p) const { return getAt(p).isBlack(); }
-	bool whiteAt(PointF p) const { return getAt(p).isWhite(); }
-	bool isEdge(PointF pos, PointF dir) const { return whiteAt(pos) && blackAt(pos + dir); }
-
 	StepResult traceStep(PointF dEdge, int maxStepSize, bool goodDirection)
 	{
 		dEdge = mainDirection(dEdge);
@@ -696,7 +574,7 @@ class EdgeTracer
 			for (int step = 1; step <= maxStepSize; ++step)
 				for (int i = 0; i <= 2*(step/4+1) * breadth; ++i) {
 					auto pEdge = p + step * d + (i&1 ? (i+1)/2 : -i/2) * dEdge;
-					log(round(pEdge));
+					log(pEdge);
 
 					if (!blackAt(pEdge + dEdge))
 						continue;
@@ -705,14 +583,14 @@ class EdgeTracer
 					for (int j = 0; j < std::max(maxStepSize, 3) && isIn(pEdge); ++j) {
 						if (whiteAt(pEdge)) {
 							// if we are not making any progress, we still have another endless loop bug
-							assert(p != round(pEdge));
-							p = round(pEdge);
+							assert(p != centered(pEdge));
+							p = centered(pEdge);
 							return StepResult::FOUND;
 						}
 						pEdge = pEdge - dEdge;
 						if (blackAt(pEdge - d))
 							pEdge = pEdge - d;
-						log(round(pEdge));
+						log(pEdge);
 					}
 					// no valid b/w border found within reasonable range
 					return StepResult::CLOSED_END;
@@ -721,47 +599,14 @@ class EdgeTracer
 	}
 
 public:
-#ifdef PRINT_DEBUG
-	static ByteMatrix _log;
-	void log(const PointI& p) const
-	{
-		if (_log.height() != image.height() || _log.width() != image.width())
-			_log = ByteMatrix(image.width(), image.height());
-		if (isIn(p))
-			_log.set(p.x, p.y, 1);
-	}
-#else
-	void log(const PointI&) const {}
-#endif
-
-	EdgeTracer(const BitMatrix& img, PointF p, PointF d) : image(img), p(p), d(d) {}
-	EdgeTracer& operator=(const EdgeTracer& other)
-	{
-		assert(&image == &other.image);
-		p = other.p;
-		d = other.d;
-		return *this;
-	}
-	EdgeTracer(const EdgeTracer&) = default;
-	~EdgeTracer() = default;
-	EdgeTracer(EdgeTracer&&) noexcept(true) = default;
-	EdgeTracer& operator=(EdgeTracer&&) = default;
-
-	bool step(int s = 1)
-	{
-		p = round(p + s * d);
-		log(p);
-		return isIn(p);
-	}
-
-	void setDirection(PointF dir) { d = dir / std::max(std::abs(dir.x), std::abs(dir.y)); }
+	using BitMatrixCursorF::BitMatrixCursor;
 
 	bool updateDirectionFromOrigin(PointF origin)
 	{
 		auto old_d = d;
 		setDirection(p - origin);
 		// if the new direction is pointing "backward", i.e. angle(new, old) > 90 deg -> break
-		if (d * old_d < 0)
+		if (dot(d, old_d) < 0)
 			return false;
 		// make sure d stays in the same quadrant to prevent an infinite loop
 		if (std::abs(d.x) == std::abs(d.y))
@@ -771,13 +616,6 @@ public:
 		return true;
 	}
 
-	PointF front() const { return d; }
-	PointF back() const { return {-d.x, -d.y}; }
-	PointF right() const { return {-d.y, d.x}; }
-	PointF left() const { return {d.y, -d.x}; }
-
-	bool isEdgeBehind() const { return isEdge(PointF(p), back()); }
-
 	bool traceLine(PointF dEdge, RegressionLine& line)
 	{
 		line.setDirectionInward(dEdge);
@@ -814,7 +652,7 @@ public:
 				// In case the 'go outward' step in traceStep lead us astray, we might end up with a line
 				// that is almost perpendicular to d. Then the back-projection below can result in an
 				// endless loop. Break if the angle between d and line is greater than 45 deg.
-				if (std::abs(normalized(d) * line.normal()) > 0.7) // thresh is approx. sin(45 deg)
+				if (std::abs(dot(normalized(d), line.normal())) > 0.7) // thresh is approx. sin(45 deg)
 					return false;
 
 				auto np = line.project(p);
@@ -823,10 +661,10 @@ public:
 				// back projected where we left off the line.
 				if (distance(np, line.project(line.points().back())) < 1)
 					np = np + d;
-				p = round(np);
+				p = centered(np);
 			}
 			else {
-				auto stepLengthInMainDir = line.points().empty() ? 0.0 : mainDirection(d) * (p - line.points().back());
+				auto stepLengthInMainDir = line.points().empty() ? 0.0 : dot(mainDirection(d), (p - line.points().back()));
 				line.add(p);
 
 				if (stepLengthInMainDir > 1) {
@@ -866,67 +704,21 @@ public:
 	bool traceCorner(PointF dir, PointF& corner)
 	{
 		step();
-		corner = PointF(p);
+		log(p);
+		corner = p;
 		std::swap(d, dir);
 		traceStep(-1 * dir, 2, false);
 #ifdef PRINT_DEBUG
-		printf("turn: %d x %d -> %.2f, %.2f\n", p.x, p.y, d.x, d.y);
+		printf("turn: %.0f x %.0f -> %.2f, %.2f\n", p.x, p.y, d.x, d.y);
 #endif
 		return isIn(corner) && isIn(p);
 	}
 };
 
-#ifdef PRINT_DEBUG
 
-ByteMatrix EdgeTracer::_log;
-
-static void log(const std::vector<PointI>& points, int color = 2)
+static DetectorResult SampleGrid(const BitMatrix& image, PointF tl, PointF bl, PointF br, PointF tr, int width, int height)
 {
-	for (auto p : points)
-		EdgeTracer::_log.set(p.x, p.y, color);
-}
-
-static void dumpDebugPPM(const BitMatrix& image, const char* fn )
-{
-	FILE *f = fopen(fn, "wb");
-
-	// Write PPM header, P5 == grey, P6 == rgb
-	fprintf(f, "P6\n%d %d\n255\n", image.width(), image.height());
-
-	// Write pixels
-	for (int y = 0; y < image.height(); ++y)
-		for (int x = 0; x < image.width(); ++x) {
-			unsigned char r, g, b;
-			r = g = b = image.get(x, y) ? 0 : 255;
-			switch (EdgeTracer::_log.get(x, y)) {
-			case 1: r = g = b = r ? 230 : 50; break;
-			case 2: r = b = 50, g = 220; break;
-			case 3: g = r = 100, b = 250; break;
-			}
-			fwrite(&r, 1, 1, f);
-			fwrite(&g, 1, 1, f);
-			fwrite(&b, 1, 1, f);
-		}
-	fclose(f);
-}
-
-static void printBitMatrix(const BitMatrix& matrix)
-{
-	for (int y = 0; y < matrix.height(); ++y) {
-		for (int x = 0; x < matrix.width(); ++x)
-			printf("%c ", matrix.get(x, y) ? '+' : '.');
-		printf("\n");
-	}
-}
-#endif
-
-static BitMatrix SampleGrid(const BitMatrix& image, PointF tl, PointF bl, PointF br, PointF tr, int dimensionX,
-							int dimensionY)
-{
-	auto moveTowardsBy = [](PointF& a, const PointF& b, double d) {
-		auto a2b = normalized(b - a);
-		a = a + d * a2b;
-	};
+	auto moveTowardsBy = [](PointF& a, PointF b, double d) { a = movedTowardsBy(a, b, d); };
 
 	// shrink shape by half a pixel to go from center of white pixel outside of code to the edge between white and black
 	moveTowardsBy(tl, br, 0.5);
@@ -936,33 +728,25 @@ static BitMatrix SampleGrid(const BitMatrix& image, PointF tl, PointF bl, PointF
 	// inclined toward the center anyway.
 	moveTowardsBy(tr, bl, 0.3);
 
-	// work around a missing 'round' in GridSampler.
-	// TODO: the correct location for the rounding is after the transformation in GridSampler
-	// but that would currently break other 2D encoders
-	for (auto* p : {&tl, &bl, &br, &tr})
-		*p = *p + PointF(0.5, 0.5);
-
-	auto border = 0.f;
-
-	return GridSampler::Instance()->sampleGrid(
-		image,
-		dimensionX, dimensionY,
-		border, border,
-		dimensionX - border, border,
-		dimensionX - border, dimensionY - border,
-		border,	dimensionY - border,
-		(float)tl.x, (float)tl.y,
-		(float)tr.x, (float)tr.y,
-		(float)br.x, (float)br.y,
-		(float)bl.x, (float)bl.y);
+	return SampleGrid(image, width, height, PerspectiveTransform(Rectangle(width, height, 0), {tl, tr, br, bl}));
 }
 
 static DetectorResult DetectNew(const BitMatrix& image, bool tryRotate)
 {
 	// walk to the left at first
-	for (auto startDirection : {PointF(-1, 0), PointF(1, 0), PointF(0, -1), PointF(0, 1)}) {
-		EdgeTracer startTracer(image, PointF(image.width()/2, image.height()/2), startDirection);
+#ifdef PRINT_DEBUG
+	LogMatrixWriter lmw(log, image, 1, "dm-log.pnm");
+	for (auto startDirection : {PointF(-1, 0)}) {
+#else
+	for (auto startDirection : {PointF(-1, 0), PointF(-1, 0), PointF(1, 0), PointF(0, -1), PointF(0, 1)}) {
+#endif
+		// TODO: If neither the horizontal nor the vertical center line cross the symbol, it will be overlooked.
+		// Will need to look at more lines.
+		auto center = PointF(image.width()/2, image.height()/2);
+		EdgeTracer startTracer(image, centered(center - center * startDirection), startDirection);
 		while (startTracer.step()) {
+			log(startTracer.p);
+
 			// go forward until we reach a white/black border
 			if (!startTracer.isEdgeBehind())
 				continue;
@@ -1047,7 +831,7 @@ static DetectorResult DetectNew(const BitMatrix& image, bool tryRotate)
 				continue;
 
 #ifdef PRINT_DEBUG
-			printf("L: %f, %f ^ %f, %f > %f, %f (%d : %d : %d : %d)\n", bl.x, bl.y,
+			printf("L: %.1f, %.1f ^ %.1f, %.1f > %.1f, %.1f (%d : %d : %d : %d)\n", bl.x, bl.y,
 			       tl.x - bl.x, tl.y - bl.y, br.x - bl.x, br.y - bl.y, (int)lenL, (int)lenB, (int)lenT, (int)lenR);
 #endif
 
@@ -1070,7 +854,7 @@ static DetectorResult DetectNew(const BitMatrix& image, bool tryRotate)
 			splitDouble(lineR.modules(br, tr), &dimR, &fracR);
 
 #ifdef PRINT_DEBUG
-			printf("L: %f, %f ^ %f, %f > %f, %f ^> %f, %f\n", bl.x, bl.y,
+			printf("L: %.1f, %.1f ^ %.1f, %.1f > %.1f, %.1f ^> %.1f, %.1f\n", bl.x, bl.y,
 			       tl.x - bl.x, tl.y - bl.y, br.x - bl.x, br.y - bl.y, tr.x, tr.y);
 			printf("dim: %d x %d\n", dimT, dimR);
 #endif
@@ -1087,22 +871,20 @@ static DetectorResult DetectNew(const BitMatrix& image, bool tryRotate)
 			if (dimT < 10 || dimT > 144 || dimR < 8 || dimR > 144 )
 				continue;
 
-			auto bits = SampleGrid(image, tl, bl, br, tr, dimT, dimR);
+			auto res = SampleGrid(image, tl, bl, br, tr, dimT, dimR);
 
 #ifdef PRINT_DEBUG
 			printf("modules top: %d, right: %d\n", dimT, dimR);
-			printBitMatrix(bits);
+			printf("%s", ToString(res.bits()).c_str());
 
 			for (RegressionLine* l : {&lineL, &lineB, &lineT, &lineR})
 				log(l->points());
-
-			dumpDebugPPM(image, "binary.pnm");
 #endif
 
-			if (bits.empty())
+			if (!res.isValid())
 				continue;
 
-			return {std::move(bits), {tl, bl, br, tr}};
+			return res;
 		}
 		// reached border of image -> try next scan direction
 #ifndef PRINT_DEBUG
@@ -1111,15 +893,51 @@ static DetectorResult DetectNew(const BitMatrix& image, bool tryRotate)
 			break; // only test left direction
 	}
 
-#ifdef PRINT_DEBUG
-	dumpDebugPPM(image, "binary.pnm");
-#endif
-
 	return {};
 }
 
-DetectorResult Detector::Detect(const BitMatrix& image, bool tryHarder, bool tryRotate)
+/**
+* This method detects a code in a "pure" image -- that is, pure monochrome image
+* which contains only an unrotated, unskewed, image of a code, with some white border
+* around it. This is a specialized method that works exceptionally fast in this special
+* case.
+*/
+static DetectorResult DetectPure(const BitMatrix& image)
 {
+	const int minSize = 8; // datamatrix codes are at least 8x8 modules
+	int left, top, width, height;
+	if (!image.findBoundingBox(left, top, width, height, minSize)) {
+		return {};
+	}
+
+	// find the first white pixel on the diagonal
+	int moduleSize = 1;
+	while (moduleSize < width / minSize && image.get(left + moduleSize, top))
+		++moduleSize;
+
+	int matrixWidth = width / moduleSize;
+	int matrixHeight = height / moduleSize;
+	if (matrixWidth < minSize || matrixHeight < minSize) {
+		return {};
+	}
+
+	// Push in the "border" by half the module width so that we start
+	// sampling in the middle of the module. Just in case the image is a
+	// little off, this will help recover.
+	int msh    = moduleSize / 2;
+	int right  = left + width - 1;
+	int bottom = top + height - 1;
+
+	// Now just read off the bits (this is a crop + subsample)
+	return {Deflate(image, matrixWidth, matrixHeight, top + msh, left + msh, moduleSize),
+			{{left, top}, {right, top}, {right, bottom}, {left, bottom}}};
+}
+
+DetectorResult Detector::Detect(const BitMatrix& image, bool tryHarder, bool tryRotate, bool isPure)
+{
+	if (isPure)
+		return DetectPure(image);
+
 	auto result = DetectNew(image, tryRotate);
 	if (!result.isValid() && tryHarder)
 		result = DetectOld(image);
